/*=============================================================================
File manager modified to use My Documents
=============================================================================*/


#include "FFileManagerWindows.h"
#include "Shlwapi.h"
#include "resource.h"
#include "misc.h"

class FFileManagerDeusExe : public FFileManagerWindows
{
	private:
	TCHAR userDataPath[MAX_PATH];
	TCHAR exePath[MAX_PATH];

	/**
	Convert file paths to 'modern' ones in. When reading, the original file is used if it exists, writing is always to appdata.
	*/
	void toModernFileName(TCHAR* newName, const TCHAR* filename, char op = 'r')
	{	
		wcscpy_s(newName,MAX_PATH,filename);
		if(!getUseUserDocs() || wcswcs(filename,userDataPath)) //User wants to use old file names after all or already modern path
		{			
			return;		
		}        
                
		if(!PathIsRelative(filename)) //Make nonrelative paths relative
		{
			PathRelativePathTo(newName,exePath,0,filename,0);			
		}

		PathCombine(newName,L"System",newName);  //The games use paths relative to system
        if(newName[0]=='\\') //Things like ..\Textures will be changed to \Textures by the above call
            PathCombine(newName,userDataPath,newName+1);
        else
            PathCombine(newName,userDataPath,newName);

		//Create directory if needed
		if(op=='w' && !PathIsDirectory(filename)) //PathIsDirectory needed as PathRemoveFileSpec would strip stuff like 'Save040' to just 'Save'
		{
			wchar_t* fileSpec = PathFindFileName(newName);
			PathRemoveFileSpec(newName);
			if(!PathFileExists(newName))
			{
				SHCreateDirectoryEx(NULL,newName,nullptr);
			}
			PathAppend(newName,fileSpec);
		}
		else if(op=='r' && !PathFileExists(newName) && PathFileExists(filename) ) //If opening a file that already exists, return original.
		{
			wcscpy_s(newName,MAX_PATH,filename);
			return;
		}
	}

public:
	FFileManagerDeusExe():FFileManagerWindows()
	{		
		getUserDocsDir(userDataPath);

		GetModuleFileName(nullptr, exePath, MAX_PATH);
		PathRemoveFileSpec(exePath);
		PathAddBackslash(exePath); //For PathRelativePathTo
	}

	FArchive* CreateFileReader( const TCHAR* _Filename, DWORD Flags, FOutputDevice* Error )
	{
		TCHAR Filename[MAX_PATH];
		toModernFileName(Filename,_Filename);
		return FFileManagerWindows::CreateFileReader(Filename,Flags,Error);
	}
	FArchive* CreateFileWriter( const TCHAR* _Filename, DWORD Flags, FOutputDevice* Error )
	{
		TCHAR Filename[MAX_PATH];
		toModernFileName(Filename,_Filename,'w');
		return FFileManagerWindows::CreateFileWriter(Filename,Flags,Error);
	}
	INT FileSize( const TCHAR* _Filename )
	{
		TCHAR Filename[MAX_PATH];
		toModernFileName(Filename,_Filename);
		return FFileManagerWindows::FileSize(Filename);
	}
	UBOOL Copy( const TCHAR* _DestFile, const TCHAR* _SrcFile, UBOOL ReplaceExisting, UBOOL EvenIfReadOnly, UBOOL Attributes, void (*Progress)(FLOAT Fraction) )
	{
		wchar_t DestFile[MAX_PATH];
		toModernFileName(DestFile,_DestFile,'w');
		wchar_t SrcFile[MAX_PATH];
		toModernFileName(SrcFile,_SrcFile);
		return FFileManagerWindows::Copy(DestFile,SrcFile,ReplaceExisting,EvenIfReadOnly,Attributes,Progress);
	}
	UBOOL Delete( const TCHAR* _Filename, UBOOL RequireExists=0, UBOOL EvenReadOnly=0 )
	{
		TCHAR Filename[MAX_PATH];
		toModernFileName(Filename,_Filename);
		return FFileManagerWindows::Delete(Filename,RequireExists,EvenReadOnly);
	}
	UBOOL Move( const TCHAR* _Dest, const TCHAR* _Src, UBOOL Replace=1, UBOOL EvenIfReadOnly=0, UBOOL Attributes=0 )
	{
		wchar_t Dest[MAX_PATH];
		toModernFileName(Dest,_Dest);
		wchar_t Src[MAX_PATH];
		toModernFileName(Src,_Src);
		return FFileManagerWindows::Move(Dest,Src,Replace,EvenIfReadOnly,Attributes);
	}
	
	UBOOL MakeDirectory( const TCHAR* _Path, UBOOL Tree=0 )
	{
		wchar_t Path[MAX_PATH];
		toModernFileName(Path,_Path,'w');
		return FFileManagerWindows::MakeDirectory(Path,Tree);
	}

	UBOOL DeleteDirectory( const TCHAR* _Path, UBOOL RequireExists=0, UBOOL Tree=0 )
	{
		wchar_t Path[MAX_PATH];
		toModernFileName(Path,_Path);
		return FFileManagerWindows::DeleteDirectory(Path,RequireExists,Tree);
	}

	TArray<FString> FindFiles( const TCHAR* _Filename, UBOOL Files, UBOOL Directories )
	{

		wchar_t Filename[MAX_PATH];
		toModernFileName(Filename,_Filename);


		guard(FFileManagerWindows::FindFiles);
		TArray<FString> Result;
		HANDLE Handle=NULL;

		
		WIN32_FIND_DATAW Data;

		//Look for old style files
		Handle=FindFirstFileW(_Filename,&Data);
		if( Handle!=INVALID_HANDLE_VALUE )
			do
				if
				(	appStricmp(Data.cFileName,TEXT("."))
				&&	appStricmp(Data.cFileName,TEXT(".."))
				&&	((Data.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)?Directories:Files) )
					new(Result)FString(Data.cFileName);
			while( FindNextFileW(Handle,&Data) );

		if( Handle )
			FindClose( Handle );
		
		if(wcscmp(Filename,_Filename)==0) //Already started out with a new style file
			return Result;

		//Look for new files
		Handle=FindFirstFileW(Filename,&Data);
		if( Handle!=INVALID_HANDLE_VALUE )
			do
				if
				(	appStricmp(Data.cFileName,TEXT("."))
				&&	appStricmp(Data.cFileName,TEXT(".."))
                &&	((Data.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)?Directories:Files) 
                && Result.FindItemIndex(Data.cFileName) == INDEX_NONE ) //Don't list things twice if the game won't be able to tell them apart (i.e. a directory like 'Save001' with no further path will result in the new version being listed twice)
                {
					new(Result)FString(Data.cFileName);
                }
			while( FindNextFileW(Handle,&Data) );
		
         if(wcscmp(Filename,_Filename)==0) //Already started out with a new style file
			return Result;

		if( Handle )
			FindClose( Handle );
		return Result;
		unguard;
		
	}
};